/* ========================================
 *
 * Copyright Cypress Semiconductor, 2012
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF Cypress Semiconductor.
 *
 * ========================================
*/
/*******************************************************************************
* File Name: main.c
*
* Version: 1.0
*
* Description:
*  This project demonstrates a WiFi board that sends up to four temperature 
*  measurements. The PSoC does the temperature measurements and communicates
*  with the WiFi board using a UART.
*
********************************************************************************
* Copyright 2012, Cypress Semiconductor Corporation. All rights reserved.
* This software is owned by Cypress Semiconductor Corporation and is protected
* by and subject to worldwide patent and copyright laws and treaties.
* Therefore, you may use this software only as provided in the license agreement
* accompanying the software package from which you obtained this software.
* CYPRESS AND ITS SUPPLIERS MAKE NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* WITH REGARD TO THIS SOFTWARE, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT,
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*******************************************************************************/

#include <device.h>
#include <stdio.h>
#include <string.h>
#include "diode.h"
#include "RTD.h"
#include "thermistor.h"
#include "thermocouple.h"

#define DEBUG_CONNECTION 0 		/* Set to 1 to print debug info about the conneciton */
#define FLASH_RESET 0 			/* Set to 1 to reset the WiFi module flash */

/* This is used to timeout when waiting for "OK >" at the end of a command and*/
/* For a CR when waiting for a complete string. This is the number of 200ms cycles
/* that are counted before the timeout. */
#define TIMEOUT_CYCLES 10 /* 10*200ms = 2sec */ 

#define LCD_NUM_COLUMNS 16

#define OFF 0
#define ON  1

#define FALSE 0
#define TRUE  1

/* Flag bits for the commands that need to send data over WiFi */
#define RT1		0x01
#define RT2		0x02
#define RT3		0x04
#define RT4		0x08
#define RTA		0x0F
	

/* Functions */
void SendCmdWait(char *cmd);
void SendCmdNoWait(char *cmd);
void WaitForOK(void);
uint8 CheckRxData(char *string);
void MeasureTemperatures(void);
void DisplayTemperatures(void);
void ProcessWiFi(void);
void ParseCommand(void);
void SendTemperature(uint8 tempNumber);

/* Global Variables */
float tempValue[4];
uint8 tempState[4] = {0, 0, 0, 0}; /* Start with all temperature reporting off */
uint8 tempSequence = 0;
uint8 rxData[0xFF]; /* This holds the receive data string */
uint8 rxIndex = 0;  /* Index for location in the receive data string */
uint8 cmdFlag = 0;  /* Flag to track which temperature needs to be sent */

/* Flag to toggle CJ selection between thermistor and IC */
uint8 CJSelectFlag=0;
	
void main()
{    	
	#define INIT 			0
	#define WAIT1 			1
	#define WAIT2			2
	#define SETUP_CLIENT 	3
	#define SETUP_SR 		4
	#define RUN 			5
	
	uint8 state=INIT;
	
	uint8 i;
	
	CyGlobalIntEnable; 

    UART_1_Start();     		/* Enabling the UART */
    LCD_Char_1_Start(); 		/* Enabling the LCD */
	LCD_Char_1_ClearDisplay();	/* Clear LCD */
	Tick_Timer_Start(); 		/* Start tick timer */

	/* Start temp sense components */
	ADCMux_Start();
	CurrentMux_Start();
	Opamp_Start();
	ADC_Start();

	
	UART_1_ClearRxBuffer(); /* Clear any bad data from WiFi module starup */

	P5_0_WiFiResetN_Write(1); /* Release reset on WiFi module */
	CyDelay(500);			/* Wait for WiFi module to startup */

	/* Measure all four temperatures once before waiting for WiFi connection */
	for(i=0; i<4; i++)
	{
		MeasureTemperatures();
	}
	
    for(;;) 
    {		
		switch(state)
		{
		case INIT: /* Initial startup */
			#if FLASH_RESET /* The next 4 lines are used to reset the flash on the WiFi module */
				SendCmdNoWait("Z3=0\r\n");
				SendCmdNoWait("Z2\r\n");
				SendCmdNoWait("ZR\r\n");
				while(1);
			#endif
			SendCmdNoWait("A0\r\n");
			state++;
			break;
		case WAIT1: /* Wait for wireless to be ready */
			if(CheckRxData("[AP IP  ] 192.168.10.1"))
			{
				LED4_Write(ON); /* Light LED and move to next stage */
				state++;
			}
			break;
		case WAIT2: /* Wait for Direct connect to be established */
			if(CheckRxData("[JOIN   ] !DIRECTCONNECT"))
			{
				WaitForOK();
				LED3_Write(ON);
				state++;
			}
			break;
		case SETUP_CLIENT: /* Start up the client */
			UART_1_ClearRxBuffer(); /* Clear any previous data from the buffer */
			SendCmdWait("P1=0\r\n");
			SendCmdWait("P3=192.168.10.100\r\n");
			SendCmdWait("P4=8001\r\n");
			#if DEBUG_CONNECTION /* Print debug information about the connection */
				SendCmdWait("$$$\r\n");
				SendCmdWait("P?\r\n");
				SendCmdWait("C?\r\n");
			#endif
			SendCmdWait("P6=1\r\n");
			CyDelay(500);
			state++;
			break;
		case SETUP_SR: /* Setup send and receive settings */
			UART_1_ClearRxBuffer(); /* Clear any previous data from the buffer */
			SendCmdWait("S1=6\r\n"); 	/* Timeout is 1000ms on a send */
			SendCmdWait("S2=250\r\n"); 	/* Timeout is 1000ms on a send */
			SendCmdWait("R1=6\r\n");	/* Receive message = 1000 bytes */
			SendCmdWait("R2=250\r\n");	/* Receive timeout */
			SendCmdWait("R3=1\r\n");		/* Supress no data message and timeout */
			state++;
			break;
		case RUN: /* Send temperature and read temperature status  */
			if(Tick_Event_Read())	/* Run this code whenever the timer expires */
			{				
				ProcessWiFi();			/* Get/send WiFi data */
			}
			break;
		default:
			break;
		}

		MeasureTemperatures();
		DisplayTemperatures();

		/* Look for reconnect switch */
		/* This is used if the TCP server is stopped */
		if(!P15_5_Reconnect_Read()) /* SW3 is pressed */
		{
			while(!P15_5_Reconnect_Read())
			{
				/* Wait for SW3 to be released */
			}
			SendCmdWait("P6=1\r\n");
			state = SETUP_SR;
		}	
    }	
}
/***********************************************************/
/* End of main loop                                        */
/***********************************************************/

/***********************************************************/
/* Functions                                               */
/***********************************************************/

/***********************************************************
* SendCmdWait
*
* Purpose: This function sends a command over the UART
* interface to the WiFi module and waits for "OK" and ">"
* to be returned.
*
* Input:
*   cmdStr: This is a string to be sent to the UART
*
***********************************************************/
void SendCmdWait(char *cmdStr)
{
	SendCmdNoWait(cmdStr);
	WaitForOK();
}

/***********************************************************
* SendCmdNoWait
*
* Purpose: This function sends a command over the UART
* interface to the WiFi module but does not wait for a
* response.
*
* Input:
*   cmdStr: This is a string to be sent to the UART
*
***********************************************************/
void SendCmdNoWait(char *cmdStr)
{

	uint8 i=0;
 
	while(cmdStr[i] != 0x00) /* Loop until end of string is reached */
    {
	   UART_1_WriteTxData(cmdStr[i]);      /* Sending the data */
	   i++;
    }
}

/***********************************************************
* WaitForOK
*
* Purpose: This function waits for "OK" and ">" to be 
* returned to the UART from the WiFi modlue. A timer
* timeout after "TIMEOUT_CYCLES" intervals so that this function
* will not lock up for failed commands that don't return "OK"
* and ">". The timer interval is 200ms so the timeout period is
* 200ms * TIMEOUT_CYCLES.
*
***********************************************************/
void WaitForOK(void)
{
	#define	START			0
	#define O				1
	#define K				2
	#define END				3
	
	uint8 state = START;
	uint8 rxTemp;
	uint8 timeout = TIMEOUT_CYCLES;
	
	while(state < END)
	{
		rxTemp = UART_1_GetChar();
		
		if (0x20 < rxTemp && rxTemp < 0x7f) /* Only capture alpha-numeric ASCII characters (excluding spaces) */
		{
			switch(state)
			{
			case START:
				if(rxTemp == 'O')
				{
					state++;
				}
				break;
			case O:
				if(rxTemp == 'K')
				{
					state++;
				}
				else
				{
					state = START;
				}
				break;
			case K:
				if(rxTemp == '>')
				{
					state++; /* This gets us to the END state which will exit the loop */
				}
				else
				{
					state = START;
				}
				break;
			}
		}
		/* Timeout so that we don't get stuck after a bad command packet */
		if(Tick_Event_Read())
		{
			timeout--;
		}
		if(!timeout)
		{
			state = END;
		}	
	}
	/* Once we exit the while loop, we have recieved "OK >" or we have timed out */
}

/***********************************************************
* CheckRxData
*
* Purpose: This function collects characters from the UART
* and compares them to the supplied string once a carridge
* return is detected.
*
* This function is blocking in that it will continue to
* wait until the expected string is received or a timeout
* occurs.
*
* Input:
*   string: This is the string to be compared against the
*   UART data
*
* Return:
*   The function returns 1 when the strings match. It will
*   return a 0 if a match is not found within the timeout
*   period.
***********************************************************/
uint8 CheckRxData(char *string)
{
	uint8 rxTemp;
	uint8 timeout = TIMEOUT_CYCLES;
	
	/* Check the UART status */
	
    while(1) /* Do this until we get the expected result */
    {
		rxTemp = UART_1_GetChar();  	
		if(rxTemp == '\r') /* We got a return - need co compare to expected string */
		{
			rxData[rxIndex] = 0x00;	/* Add null termination at end of string */
			rxIndex = 0; 			/* Reset index for next string */
			if(!strcmp(string, rxData)) /* See if we got the expected sring */
			{
				return 1;
			}
		}
		else
		{
		    if (0x1f < rxTemp && rxTemp < 0x7f) /* This is an alpha-numeric char (including spaces) */
			{
				rxData[rxIndex] = rxTemp;
				rxIndex++;
			}
		}
		/* Timeout so that we don't get stuck after a bad command packet */
		if(Tick_Event_Read())
		{
			timeout--;
		}
		if(!timeout)
		{
			return 0;
		}
	}
	return 0; /* We will never get to this line */
}

/***********************************************************
* MeasureTemperatures
*
* Purpose: This function measures the four temperatures and
* stores them in an array. One temperature is measured on
* each pass through the function in order to minimize the
* delay to the firmware loop.
*
***********************************************************/
void MeasureTemperatures(void)
{
	switch(tempSequence)
	{
	case 0:
		tempValue[0] = MeasureRTDTemp();
		tempSequence++;
		break;
	case 1:
		tempValue[1] = MeasureThermoCoupleTemp();
		tempSequence++;
		break;
	case 2:
		tempValue[2] = MeasureThermistorTemp();
		tempSequence++;
	case 3:
		tempValue[3] = MeasureFirstDiodeTemp();
		tempSequence = 0;
	}
}

/***********************************************************
* DisplayTemperatures
*
* Purpose: This function displays the four temperature values
* to the LCD on the 030 board. If a given temperature is 
* enabled by the PC application, then an "*" will be displayed
* next to the temperature number.
*
***********************************************************/
void DisplayTemperatures(void)
{
	uint8 i;
	char8 number[0xFFu]; /* array to hold ascii representation of temperaure */
	uint8 bytesWritten;  /* number of bytes in the temperature string */
	
	for(i=0; i<4; i++)
	{
		bytesWritten = sprintf(number, "%.1f", tempValue[i]); /* convert temperature to ASCII */

		switch(i)	/* Set location on LCD */
		{
		case 0:
			LCD_Char_1_Position(0,0);
			break;
		case 1:
			LCD_Char_1_Position(0,9);
			break;
		case 2:
			LCD_Char_1_Position(1,0);
			break;
		case 3:
			LCD_Char_1_Position(1,9);
			break;
		}
		LCD_Char_1_PutChar(i+'1'); /* Print temperature number */
		if(tempState[i]) /* This temperature is being sent */
		{
			LCD_Char_1_PutChar('*'); /* Print * if being sent */
		}
		else
		{
			LCD_Char_1_PutChar(' '); /* Print space if not being sent */
		}
		LCD_Char_1_PutChar(':'); /* Print colon */
		if(bytesWritten < 4) /* Print leading space if temperature is not 2 digits */
		{
			LCD_Char_1_PutChar(' ');
		}
		LCD_Char_1_PrintString(number); /* Print temperature */
	}
}

/***********************************************************
* ProcessWiFi
*
* Purpose: This function sends an R0 command to the WiFi
* module and then looks for data to be returned. If data
* is returned it will parse the data for commands and will
* respond to them.
*
***********************************************************/
void ProcessWiFi(void)
{
	uint8 dataAvailable = TRUE;
	uint8 rxTemp;
	uint8 i;
	
	/* Send command to receive data from WiFi module */
	SendCmdNoWait("R0\r\n");

	rxIndex = 0; /* Start a new string */	
	cmdFlag=0;   /* Reset command flag */
	
	CyDelay(10); /* Wait to make sure response has come back from the WiFi module */
	
	while(dataAvailable) /* Do this until we have a full packet to process */
    {
		rxTemp = UART_1_GetChar();  /* Get a byte */
		
		if(rxTemp == 0) /* We have reached the end of the string */
		{
			dataAvailable = FALSE;
		}
		else if (('0' <= rxTemp && rxTemp <= '9') || ('A' <= rxTemp && rxTemp <= 'Z')) /* Only capture numbers and uppercase letters (ASCII characters) */
		{
			rxData[rxIndex] = rxTemp;
			rxIndex++;
		}
		else if (rxTemp == '\r') /* We have reached a carridge return - need to parse command */
		{
			ParseCommand();
			rxIndex = 0; /* reset array for next command */
			memset(rxData, 0 , 0xff);
		}
	}

	/* Now we need to actually send any requested temperatures */
	for(i=0;i<4;i++)
	{
		if((0x01<<i) & cmdFlag) /* Check if flag bit is set for this temperature */
		{
			SendTemperature(i+1);
			CyDelay(10);
		}
	}
	 
}

/***********************************************************
* ParseCommand
*
* Purpose: This function parses the command string and
* either executes the command or sets a flag for sending
* temperatures.
*
* The supported commands are:
* RT[1234A] - send temperature 1, 2, 3, 4, or all.
* T[1234]ON - turn on temperature 1, 2, 3, or 4.
* T[1234]OFF - turn off temperature 1, 2, 3, or 4.
*
* The temperature flag bits are:
* 0x01 - Send temperature 1
* 0x02 - Send temperature 2
* 0x04 - Send temperature 3
* 0x08 - Send temperature 4
*
***********************************************************/
void ParseCommand()
{	
	uint8 temperatureIndex;
	
	switch(rxData[0]) /* Look at first character in the string */
	{
	case 'R': /* Server is asking for a temperature */
		if(rxData[1] == 'T') /* 2nd character must be T */
		{
			switch(rxData[2]) /* 3rd character in the command */
			{
			case '1': /* Send temperature 1 */
				cmdFlag |= RT1;
				break;
			case '2': /* Send temperature 2 */
				cmdFlag |= RT2;
				break;
			case '3': /* Send temperature 3 */
				cmdFlag |= RT3;
				break;
			case '4': /* Send temperature 4 */
				cmdFlag |= RT4;
				break;
			case 'A': /* Send all temperatures */
				cmdFlag |= RTA;
				break;
			default: /* Not a valid command */
				break;
			}
		}
		break; /* End of case 'R' */
	case 'T': /* Servier is turning temperautre reporting on or off */
		switch(rxData[1])
		{
		case '1':
			temperatureIndex = 0;
			break;
		case '2':
			temperatureIndex = 1;
			break;
		case '3':
			temperatureIndex = 2;
			break;
		case '4':
			temperatureIndex = 3;
			break;
		default:
			/* not a valid command - need to go onto next character */
			temperatureIndex = 99;
			break;
		}
		if(temperatureIndex < 4)
		{
			if(rxData[2] == 'O')
			{
				if(rxData[3] == 'N') /* Turn on temperature reporting */
				{
					tempState[temperatureIndex] = 1;
				}
				else if(rxData[3] == 'F')
				{
					if(rxData[4] == 'F') /* Turn off temperature reporting */
					{
						tempState[temperatureIndex] = 0;
					}
				}
			}
		}
		break; /* End of case 'T' */
	default:
		break;
	} /* End of switch for parsing input stream */
}

/***********************************************************
* SendTemperature
*
* Purpose: This function sends a temperature value to the
* WiFi module. The S1 command is used to set the packet
* size and then the S0 command is used to actually send the
* data.
*
* Input:
*   tempNumber: the number of the temperature sensor whose
*   data will be sent (1, 2, 3, or 4).
***********************************************************/
void SendTemperature(uint8 tempNumber)
{
	uint8 i;
	char8 number[0xFFu]; /* array to hold ascii representation of temperaure */
	uint8 bytesWritten;  /* number of bytes in the temperature string */	
	uint8 byteString;	 /* This is the number of characters in temperature string plus T and sensor number */
	
	bytesWritten = sprintf(number, "%3.1f", tempValue[tempNumber-1]); /* convert temperature to ASCII */
	byteString = bytesWritten+2;
	
	/* Set up number of bytes to send : T + # + temperature chars */
	SendCmdNoWait("S1="); 
	
	
	if(byteString < 10) /* If string to be sent is less than 10 (i.e. 1 digit) */
	{
		UART_1_WriteTxData(byteString+'0');
	}
	else if(byteString < 20) /* If string is 10 - 19 */
	{
		UART_1_WriteTxData('1');
		UART_1_WriteTxData(byteString-10+'0');
	}
	else /* Assume string is 20 - 29 */
	{
		UART_1_WriteTxData('2');
		UART_1_WriteTxData(byteString-20+'0');	
	}
	SendCmdWait("\r\n");

	/* Send temperature */
	SendCmdNoWait("S0\r");
	UART_1_WriteTxData('T');
	UART_1_WriteTxData(tempNumber+'0'); /* Temperautre number (1-4) in ASCII */
	for(i=0; i<bytesWritten; i++) /* Send temperature bytes */
	{
		UART_1_WriteTxData(number[i]);
	}
	UART_1_WriteTxData('\r');
	SendCmdWait("\r\n");
}
