/******************************************************************************
* File Name: thermistor.c
*  Version 2.1
*
* Description:
*  This file contains code for temperature measurement using thermistor sensor
*
* Note:
*
********************************************************************************
* Copyright (2012), Cypress Semiconductor Corporation. All Rights Reserved.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*******************************************************************************/
#include "thermistor.h"


/*******************************************************************************
* Function Name:  MeasureThermistorTemp
********************************************************************************
* Summary:
* This functions finds the thermistor temperature using two sub functions 
* ThermistorGetResistance() and ThermistorRestoTemp()
* 
* Parameters:  
* void
*
* Return: 
* float: Temperature in °C

* 
* Side Effects:
* None 	
*******************************************************************************/
float MeasureThermistorTemp(void)
{
	/* Thermistor resistance */
	float thermResistance;
	
	/*Thermistor temperature */
	float thermTemp;
	
	thermResistance = ThermistorGetResistance();
	
	thermTemp = ThermistorRestoTemp(thermResistance);
       
    return thermTemp;
}

/*******************************************************************************
* Function Name:  ThermistorGetResistance
********************************************************************************
* Summary:
* This function reads the voltage across the thermistor and voltage across 
* reference resistor. This two voltage value are used to calculate thermistor 
* resistance.
* 
* Parameters:  
* void:  
*
* Return: 
* float: Resistance in ohms

* Theory: 
* Thermistor resistance is calculated by using the following equation:
* Rthermistor = Rref * Vsignal / Vref 
*
* Side Effects:
* None 	
*******************************************************************************/
float ThermistorGetResistance(void)
{

	/* Variable used to calculate VSSA ground reference voltage */
    int32 gndRefOffset = 0;
    
    /* Variable to store voltage read across reference resistor */
    int32 thermRefResult = 0;
    
    /* Variable to store voltage across thermistor */
    int32 thermSignalResult = 0;

    /* Calculated thermistor resistance */
    float thermResistance = 0.0;
    
    /* Read ground reference voltage */
    gndRefOffset = GetVssaRefSample();
       
    /* Read voltage across reference resistor */
    ADCMux_FastSelect(THERM_REF_CHANNEL);
    ADC_StartConvert();
    ADC_IsEndConversion(ADC_WAIT_FOR_RESULT);
    ADC_StopConvert();
    thermRefResult = ADC_GetResult32();
    
    /* get the CDS difference( Thermistor ref signal voltage is inverted because
     * this signal is connected to -ve input of ADC mux and ground is connected
     * to +ve input of mux ) */    
	thermRefResult = gndRefOffset - thermRefResult;
         
    /* Read voltage across thermistor */
    ADCMux_FastSelect(THERM_SIGNAL_CHANNEL);
    ADC_StartConvert();
    ADC_IsEndConversion(ADC_WAIT_FOR_RESULT);
    ADC_StopConvert();
    thermSignalResult = ADC_GetResult32();
       
	/* Get CDS difference */
    thermSignalResult = thermSignalResult - gndRefOffset;
	
	/* Rthermistor = Rref * Vsignal / Vref */
	thermResistance = (thermSignalResult * THERMISTOR_REF_RES) / thermRefResult;
	
	return thermResistance;
	
}	

/*******************************************************************************
* Function Name:  ThermistorRestoTemp
********************************************************************************
* Summary:
* This function gets the thermistor resistance value and converts it into 
* temperature using Steinhart-Hart equation.
* 
* Parameters:  
* float: Resistance in ohms  
*
* Return: 
* float: Temperature in °C

* Theory: 
* The resistance value of the thermistor is used to calculate the temperature
* using following Steinhart-Hart equation.
* 1/T = A + B*ln(R) + C*ln(R)^3
*
* Side Effects:
* None 	
*******************************************************************************/
float ThermistorRestoTemp(float thermResistance)
{	
	float thermTemp;
	
	float thermLogR;

	thermLogR = log(thermResistance);
	
	/* Steinhart-Hart equation */
    thermTemp = (1 / (STEIN_CO_A + (STEIN_CO_B * thermLogR) + (STEIN_CO_C * pow(thermLogR , 3.0))));
    
    /* Convert temperature in kelvin to degree celsius */
    thermTemp -= ZERO_DEGREE_CELSIUS_KELVIN;
	
	return thermTemp;
}

/* [] END OF FILE */

