/******************************************************************************
* File Name: thermocouple.c
*  Version 2.1
*
* Description:
*  This file contains code for temperature measurement using thermocouple
*
* Note:
*
********************************************************************************
* Copyright (2012), Cypress Semiconductor Corporation. All Rights Reserved.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*******************************************************************************/
#include "thermocouple.h"
   
/* calibration constant for cold junction compensation IC */				  
float cjCalibrationConst; 		

/*******************************************************************************
* Function Name:  MeasureThermocoupleTemp
********************************************************************************
* Summary:
* This function measures the temperature using thermocouple. First, the cold 
* junction temperature is measured using DS600 IC and equivalent voltage for the 
* thermocouple for this temperature is found. After that, the hot junction
* voltage is measured and this voltage is added to cold junction thermocouple 
* voltage. This total voltage is then converted to temperature using thermocouple
* temperature coefficient.
*
* Parameters:  
* void:  
*
* Return: 
* float: Temperature in °C

* Theory: 
* None
* 
* Side Effects:
* None	
*******************************************************************************/
float MeasureThermocoupleTemp(void)
{
    /* VSSA reference voltage */
    int32 gndRefOffset = 0;
    
    /* Thermocouple voltage calculated from the cold junction temperature */
    float tcColdJnmVolt = 0.0;
     
    /* TC hot junction ADC count measurement */
    int32 hotJnResult = 0;
    
    /* Filtered hot junction count */
    static float fHotJnCount = 0.0;
    
    /* Thermocouple volt calculated from ADC hot Jn count */
    float tcHotJnmVolt = 0;
    
    /* Thermocouple voltage measured from tcCount 
       (cold junction voltage + hot junction voltage) */
    float tcmVolt = 0;
    
    /* Flag used for filter, if TRUE ADC reading is not filtered */
    static uint8 firstTcSample = TRUE; 
    static float prevTcTemp = 0.0;
    
    float tcTemp = 0.0;
    float coldJnTemp = 0.0;
    
    /* Measure cold junction temperature */
    coldJnTemp = MeasureColdJnSensorTemp();
    
    /* Cold junction temp is processed here to get TC equivalent uVolt */
    tcColdJnmVolt = ColdJnTempTomVolts(coldJnTemp);
    
    /* Read ground reference voltage */
    gndRefOffset = GetVssaRefSample();

    /* Read thermocouple hot junction voltage */
    ADCMux_FastSelect(TC_HOT_JN_CHANNEL);
    ADC_StartConvert();
    ADC_IsEndConversion(ADC_WAIT_FOR_RESULT);
    ADC_StopConvert();
    hotJnResult = ADC_GetResult32();
    hotJnResult = hotJnResult - gndRefOffset;
    
    /* Filter signal */
    fHotJnCount = FilterSignal(firstTcSample, hotJnResult, fHotJnCount, TC_FILTER_COEFF);
    firstTcSample = FALSE;

    tcHotJnmVolt = ADC_CountsTo_uVolts((int32)fHotJnCount) / 1000.0;
    tcmVolt = tcColdJnmVolt + tcHotJnmVolt;
    tcTemp = TcmVoltsToTemp(tcmVolt);
    
    /* Reset the filter if previous reading and current reading is beyond 
     * threshold, This is done to make the filter response sharp */
    if( ( tcTemp > (prevTcTemp + FILTER_RESET ) ) || 
        ( tcTemp < (prevTcTemp - FILTER_RESET ) ) )
    {
        firstTcSample = TRUE;
    }
    prevTcTemp = tcTemp;
    
    return tcTemp;
}
/*******************************************************************************
* Function Name:  TcmVoltsToTemp
********************************************************************************
* Summary:
* This function takes the thermocouple micro volts as input and calculates temp-
* erature by finding the coefficient for the corresponding micro volts using the
* NIST polynomial. (Maximum error <= 0.08 °C)
*  T = A9 * V^9 + A8* V^8 + A7 * V^7 + A6 * V^6 + A5 * V^5 + A4 * V^4 + A3 * V^3 
*      + A2 * V^2 + A1 * V + A0
* 
* Parameters:  
* int32:  tcmVolt( TC output milli volts(cold junction + hot junction )
*
* Return: 
* float: Temperature in °C
*  
* Side Effects:
* None 	
*******************************************************************************/
float TcmVoltsToTemp(float tcmVolt)
{
  float tcTemp;
  /* Polynomial coefficients for converting thermocouple voltage to temperature */
  CYCODE const float coeffV2T[][4] = { -3261185.528,  0.0000000E+00,  0.000000E+00, -1.318058E+02,
			                            0          ,  2.5173462E+01,  2.508355E+01,  4.830222E+01,
			                            0          , -1.1662878E+00,  7.860106E-02, -1.646031E+00,
			                           -485779.8806, -1.0833638E+00, -2.503131E-01,  5.464731E-02,
			                           -179536.7035, -8.9773540E-01,  8.315270E-02, -9.650715E-04,
			                            0          , -3.7342377E-01, -1.228034E-02,  8.802193E-06,
			                            10763.10133, -8.6632643E-02,  9.804036E-04, -3.110810E-08,
			                            2272.745917, -1.0450598E-02, -4.413030E-05,  0.000000E+00,
			                            195.9575192, -5.1920577E-04,  1.057734E-06,  0.000000E+00,
			                            6.356725543,  0.0000000E+00, -1.052755E-08,  0.000000E+00};
		
	/*  T = A9 * V^9 + A8* V^8 + A7 * V^7 + A6 * V^6 + A5 * V^5 + A4 * V^4 + A3 * V^3 + A2 * V^2 + A1 * V + A0 */
	if (tcmVolt < -5.891)
	{
	    tcTemp = (((((((((coeffV2T[9][0] * tcmVolt + coeffV2T[8][0]) * tcmVolt + 
						  coeffV2T[7][0]) * tcmVolt + coeffV2T[6][0]) * tcmVolt + 
						  coeffV2T[5][0]) * tcmVolt + coeffV2T[4][0]) * tcmVolt + 
				 		  coeffV2T[3][0]) * tcmVolt + coeffV2T[2][0]) * tcmVolt + 
						  coeffV2T[1][0]) * tcmVolt + coeffV2T[0][0]);
	}
	else if (tcmVolt <= 0)
	{
		tcTemp = (((((((((coeffV2T[9][1] * tcmVolt + coeffV2T[8][1]) * tcmVolt + 
						  coeffV2T[7][1]) * tcmVolt + coeffV2T[6][1]) * tcmVolt + 
						  coeffV2T[5][1]) * tcmVolt + coeffV2T[4][1]) * tcmVolt + 
						  coeffV2T[3][1]) * tcmVolt + coeffV2T[2][1]) * tcmVolt + 
						  coeffV2T[1][1]) * tcmVolt + coeffV2T[0][1]);
	}
	else if(tcmVolt <= 20.644)
	{
	    tcTemp = (((((((((coeffV2T[9][2] * tcmVolt + coeffV2T[8][2]) * tcmVolt + 
						  coeffV2T[7][2]) * tcmVolt + coeffV2T[6][2]) * tcmVolt + 
						  coeffV2T[5][2]) * tcmVolt + coeffV2T[4][2]) * tcmVolt + 
						  coeffV2T[3][2]) * tcmVolt + coeffV2T[2][2]) * tcmVolt + 
						  coeffV2T[1][2]) * tcmVolt + coeffV2T[0][2]);
	}
	else
	{
	    tcTemp = (((((((((coeffV2T[9][3] * tcmVolt + coeffV2T[8][3]) * tcmVolt + 
						  coeffV2T[7][3]) * tcmVolt + coeffV2T[6][3]) * tcmVolt + 
						  coeffV2T[5][3]) * tcmVolt + coeffV2T[4][3]) * tcmVolt + 
						  coeffV2T[3][3]) * tcmVolt + coeffV2T[2][3]) * tcmVolt + 
						  coeffV2T[1][3]) * tcmVolt + coeffV2T[0][3]);
	}
    return tcTemp;
}

/*******************************************************************************
* Function Name:  ColdJnTempTomVolts
********************************************************************************
* Summary:
*  This function takes the cold junction temperature as input and calculates 
*  microvolt which is equivalent to thermocouple voltage for the given 
*  temperature using polynomial( maximum error <= 0.001mV).
*	V = A8 * T^8 + A7 * T^7 + A6 * T^6 + A5 * T^5 + A4 * T^4 + A3 * T^3 + A2 * T^2 
*        + A1 * T + A0
*
* Parameters:  
* float: cjTemp( CJ temperature )
*
* Return: 
* float:microvolts

* 
* Side Effects:
* None 	
*******************************************************************************/
float ColdJnTempTomVolts(float cjTemp)
{
	float tcColdJnmVolt;
	/* Polynomial coefficients for converting cold junction temperature to voltage*/
	CYCODE const float coeffT2V[] = {1.9757088776E-17,
									-7.3731358549E-15,
									 1.0997901353E-12,
									-8.2536386992E-11,
									 2.9532154859E-09,
									-1.2221985460E-07,
									 2.4264331137E-05,
									 3.9456886214E-02,
									 1.4611596938E-05};
	
	/*V = A8 * T^8 + A7 * T^7 + A6 * T^6 + A5 * T^5 + A4 * T^4 + A3 * T^3 + A2 * T^2 + A1 * T + A0 */
	tcColdJnmVolt = (((((((((coeffT2V[0]) * cjTemp + coeffT2V[1]) * cjTemp + coeffT2V[2]) * cjTemp + 
							 coeffT2V[3]) * cjTemp + coeffT2V[4]) * cjTemp + coeffT2V[5]) * cjTemp + 
							 coeffT2V[6]) * cjTemp + coeffT2V[7]) * cjTemp + coeffT2V[8]);
    
    return tcColdJnmVolt; 
}

/*******************************************************************************
*  Function Name: MeasureColdJnSensorTemp
********************************************************************************
* Summary:
*  This function is used to select cold junction temperature sensor which is used 
*  as cold junction compensation for thermocouple. THe selection is made based on 
*  the value of a flag
*
* Parameters:  
* void:
*
* Return: 
* float: Temperature in °C
*
* Theory:
*
* Side Effects:
* None
*******************************************************************************/
float MeasureColdJnSensorTemp(void)
{
    /* Cold Jn temperature measured using either DS600 IC or Thermistor based on switch press */
   	float coldJnTemp = 0.0;
    
   	if (CJSelectFlag == 0)
   		coldJnTemp = MeasureICTemp();
	else
		coldJnTemp = MeasureThermistorTemp();
    
	return coldJnTemp;
}

/*******************************************************************************/


/*******************************************************************************
*  Function Name: MeasureICTemp
********************************************************************************
* Summary:
*  This function is used to measure the DS600 IC temperature which is used as
*  cold junction compensation for thermocouple.
*
* Parameters:  
* void:
*
* Return: 
* float: Temperature in °C
*
* Theory:
* Cold Junction IC DS600 has temperature coefficient of 6.45mV/°C with 
* 509mv offset at °C.
* Analog voltage output from IC is converted to temperature as follows:
* Temp = (Vout - 509mV)/6.45mV
*
* Side Effects:
* None
*******************************************************************************/
float MeasureICTemp(void)
{
    /* VSSA reference voltage */
    int32 gndRefOffset = 0;
    
    /* Cold junction IC ADC count measurement*/
    int32 ICResult = 0;
   
    /* Cold Junction micro volts converted from ADC reading */
    int32 ICUVolts = 0;
    
    /* Cold Jn temperature measured using DS600 */
    float ICTemp = 0.00;
    
    /* Read ground reference voltage */
    gndRefOffset = GetVssaRefSample();

    /* Read cold junction voltage */
    ADCMux_FastSelect(IC_CHANNEL);
    ADC_StartConvert();
    ADC_IsEndConversion(ADC_WAIT_FOR_RESULT);
    ADC_StopConvert();
    ICResult = ADC_GetResult32();
    ICResult = ICResult - gndRefOffset;
    ICUVolts = ADC_CountsTo_uVolts(ICResult);
    
    /* Measure cold junction temperature
     * Temperature in Celsius from parameters from the DS600 Temp Sensor
     * 509 mV at 0 degrees C, divided by 6.45 mV per degree C.
	 * T = (V - 509)  / 6.45
	 */
    ICTemp = ((ICUVolts / 1000.0) - ZERO_DEGREE_OFFSET_VOLTAGE) / VOLTAGE_PER_DEGREE;
	
	/* subtract the CJ offset calibration constant */
	ICTemp = ICTemp - cjCalibrationConst;

    return ICTemp;
}

/* [] END OF FILE */
